/////////////////////////////////////////////////////////////////////////////////
/// FILE:     ta.cpp	 						      ///
/// CONTAINS: Implementations for TA problem classes	    		      ///
/// AUTHOR:   Manuel Vallejo Snchez 		07/11/2000		      ///
/////////////////////////////////////////////////////////////////////////////////


// CLASS IMPLEMENTATION

TA::TA ()
{
}
		
TA::~TA ()
{
	delete [] Term;
	delete [] Term_Weight;
	delete [] Conc;
	delete [] Conc_Weight;
	delete [] Cost;
}
	
	
			
double TA:: Evaluate  (double population [], unsigned long len)
{
double fitness=0.0,cost_sum,penalty_sum=0.0;
bool feasible=TRUE;
int index,overload;
 
 	fitness_counter++;		

	// Is it a feasible TA?
	for (index=0;index<conc_cardinal;index++)
		Conc_Weight[index]=0;
	
	for (index=0;index<len;index++)
		Conc_Weight[(int) population[index]]+= Term_Weight[index];
		
	overload=0;
	for (index=0;index<conc_cardinal;index++)
		if (Conc_Weight[index]>conc_capacity)
			{
			feasible=FALSE;
			overload++;
			}
			
	// Compute cost_sum
	cost_sum=0.0;
	for (index=0;index<term_cardinal;index++)
		cost_sum+= (double) Cost[index*conc_cardinal+((int) population[index])-1];
	
	
	// If necesary compute penalty
	penalty_sum=0.0;
	if (!feasible)
		{
		for (index=0;index<term_cardinal;index++)
			if ((Conc_Weight[index]-conc_capacity)>0) 
		 		penalty_sum+= (double) (Conc_Weight[index]-conc_capacity);
		penalty_sum *= (double) overload;
		}

	
	// Compute fitness
			
	if (feasible) fitness=(double) (1/cost_sum);
		else fitness=(double) (1/(cost_sum+penalty_sum));
		
 	return (fitness*10000.0); 
}

bool TA::Get_Configuration_File (char *file_name)
{
FILE *file;
int index,num;
char buffer[256],caracter;
int term_flag=0,conc_flag=0,cap_flag=0;
int i,j;
double term1,term2;

	if ((file=fopen(file_name,"rt"))==NULL)
		{
		cerr << "TA> ERROR! File not found: " << file_name << endl;
		exit(-1);
		}


	while (!feof(file))
	  {
	  fscanf(file,"%s", &buffer);
	  if (!strcmp(buffer,STR_TERMINALS))
	  		{
	  		term_flag=1;
			fscanf(file,"%d", &term_cardinal);
			Term = new short int [term_cardinal*2];
			Term_Weight = new short int [term_cardinal];
			index=0;
			while (index<term_cardinal)
				{
				fscanf(file,"%d", &num);
				Term[2*index]=num;
				fscanf(file,"%d", &num);
				Term[2*index+1]=num;
				fscanf(file,"%d", &num);
				Term_Weight[index++]=num;
				}
			fscanf(file,"%s", &buffer);
			}
	  if (!strcmp(buffer,STR_CONCENTRATORS))
			{
			conc_flag=1;
			fscanf(file,"%d", &conc_cardinal);
			Conc = new short int [conc_cardinal*2];
			Conc_Weight = new short int [conc_cardinal];
			index=0;
			while (index<conc_cardinal)
				{
				fscanf(file,"%s", &buffer);
				Conc[2*index]=atoi(buffer);
				fscanf(file,"%d", &num);
				Conc[2*index+1]=num;
				index++;
				}
			fscanf(file,"%s", &buffer);
			}
	  if (!strcmp(buffer,STR_CONCENTRATORS_CAPACITY))
	  		{
	  		cap_flag=1;
	  		fscanf(file,"%d", &conc_capacity);
			fscanf(file,"%s", &buffer);
			}
	 }
		
	
		
	if (term_flag==0) 
		{
		cerr << "TA> Error! Missing Terminal location in problem configuration file." << endl;
		exit(-1);
		}
	if (conc_flag==0) 
		{
		cerr << "TA> Error! Missing Concentrators location in problem configuration file." << endl;
		exit(-1);
		}
	if (term_flag==0) 
		{
		cerr << "TA> Error! Missing Concentrators capacity in problem configuration file." << endl;
		exit(-1);
		}


	// Compute cost to assign terminals to concentrators
	 
	Cost = new int [term_cardinal*conc_cardinal];
	if (Cost==NULL)
		{
		cerr << "TA> Error! Not enought memory in TA constructor." << endl;
		exit (-1);
		}
		
	for (i=0;i<term_cardinal*conc_cardinal;i++)
		Cost[i]=0;
		
	//Compute cost matrix
	for (i=0;i<term_cardinal;i++)
		for (j=0;j<conc_cardinal;j++)
			{
			term1= (double) (Term[i*2]-Conc[j*2]);
			term2= (double) (Term[i*2+1]-Conc[j*2+1]);
			Cost [i*conc_cardinal+j-1] = (int) sqrt (term1*term1+term2*term2);
			}	
			
	return TRUE;
}


